--
-- JohnDeereHud
-- Specialization for JohnDeereHud
--
-- @author      Karol Gruszczyk (MySQly)
-- @version     v1.0
-- @date        06/01/17
-- @history:    v1.0 - Initial version
--

JohnDeereHud = {};

function JohnDeereHud.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Steerable, specializations) and SpecializationUtil.hasSpecialization(Fillable, specializations)
end;

function JohnDeereHud:load(xmlFile)
    self.oilTemp = 0.1
    self.oilGainScale = 0.00001

    self.loadLevel = 0.0

    self.previousFillRatio = 0.0
    self.fillLevelSamplePlayed = false

    self.forwardIcon = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.JohnDeereHud.forwardIcon#node"))
    self.backwardIcon = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.JohnDeereHud.backwardIcon#node"))
    self.parkIcon = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.JohnDeereHud.parkIcon#node"))
    self.turnedOffIcons = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.JohnDeereHud.turnedOffIcons#node"))
    self.beaconIcon = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.JohnDeereHud.beaconIcon#node"))

    self.fuelIcon = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.JohnDeereHud.fuel#icon"))
    self.fuelLevelAnim = getXMLString(self.xmlFile, "vehicle.JohnDeereHud.fuel#animName")

    self.oilTempAnim = getXMLString(self.xmlFile, "vehicle.JohnDeereHud.oilTemp#animName")
    
    self.loadLevelAnim = getXMLString(self.xmlFile, "vehicle.JohnDeereHud.loadLevel#animName")

    self.joystickAnim = getXMLString(self.xmlFile, "vehicle.JohnDeereHud.joystick#animName")

    self.fillLevelAnim = getXMLString(self.xmlFile, "vehicle.JohnDeereHud.fillLevel#animName")
    self.fillLevelIcon = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.JohnDeereHud.fillLevel#icon"))

    local filename = getXMLString(self.xmlFile, "vehicle.JohnDeereHud.warningSound#filename");
    local SoundFile = Utils.getFilename(filename, self.baseDirectory);
    self.warningSound = createSample("warningSound");
    loadSample(self.warningSound, SoundFile, false);
end;

function JohnDeereHud:delete()
    if self.warningSound ~= nil then
        delete(self.warningSound);
    end;
end;

function JohnDeereHud:readStream(streamId, connection)
end;

function JohnDeereHud:writeStream(streamId, connection)
end;

function JohnDeereHud:mouseEvent(posX, posY, isDown, isUp, button)
end;

function JohnDeereHud:keyEvent(unicode, sym, modifier, isDown)
end;

function JohnDeereHud:update(dt)
    if self.isMotorStarted then
        if self.oilTemp < 0.7 then
            self.oilTemp = self.oilTemp + dt * self.oilGainScale
        end
    else
        if self.oilTemp > 0.1 then
            self.oilTemp = self.oilTemp - dt * self.oilGainScale
        end
    end
        
    if self.isEntered then
        local speed = self.movingDirection * self.lastSpeed * self.speedDisplayScale * 3600
        local scaledSpeed = self.lastSpeed * 3600

        if scaledSpeed < 0.1 then
            self:setAnimationTime(self.joystickAnim, 0.5)
        else
            if self.movingDirection > 0.0 then
                self:setAnimationTime(self.joystickAnim, 0.5 + scaledSpeed / 26.0 / 2.0)
            else
                self:setAnimationTime(self.joystickAnim, 0.5 - scaledSpeed / 16.0 / 2.0)
            end
        end

        setVisibility(self.forwardIcon, speed > 1.0)
        setVisibility(self.backwardIcon, speed < -1.0)
        setVisibility(self.parkIcon, math.abs(speed) <= 1.0)
        setVisibility(self.turnedOffIcons, not self.isMotorStarted)
        setVisibility(self.beaconIcon, self.beaconLightsActive)

        if self.isMotorStarted then
            self:setAnimationTime(self.oilTempAnim, self.oilTemp)

            local fuelRatio = self.fuelFillLevel / self.fuelCapacity
            if not getVisibility(self.fuelIcon) and fuelRatio < 0.1 then
                playSample(self.warningSound, 1, 1.0, 0.0);
            end
            setVisibility(self.fuelIcon, fuelRatio < 0.1)
            self:setAnimationTime(self.fuelLevelAnim, fuelRatio)

            local fillLevelInformations = {}
            local fillRatio = 0.0
            self:getFillLevelInformation(fillLevelInformations)
            for i, fillLevelInformation in pairs(fillLevelInformations) do
                fillRatio = fillLevelInformation.fillLevel / fillLevelInformation.capacity
                if self.previousFillRatio == 0.0 then
                    self.previousFillRatio = fillRatio
                end
                if self.previousFillRatio < 0.5 and fillRatio >= 0.5 or self.previousFillRatio < 0.8 and fillRatio >= 0.8 then
                    playSample(self.warningSound, 1, 1.0, 0.0);
                end
                if fillRatio < self.previousFillRatio then
                    if not self.fillLevelSamplePlayed then
                        playSample(self.warningSound, 1, 1.0, 0.0);
                        self.fillLevelSamplePlayed = true
                    end
                else
                    self.fillLevelSamplePlayed = false
                end
                local currentLoadLevel = math.min((fillRatio - self.previousFillRatio) * dt * 150.0, 1.0)
                self.loadLevel = self.loadLevel + (currentLoadLevel - self.loadLevel) / 30.0
                self:setAnimationTime(self.loadLevelAnim, self.loadLevel)

                self.previousFillRatio = fillRatio
            end
            setVisibility(self.fillLevelIcon, fillRatio > 0.8)
            self:setAnimationTime(self.fillLevelAnim, fillRatio)
        else
            self:setAnimationTime(self.fuelLevelAnim, 0.0)
            self:setAnimationTime(self.fillLevelAnim, 0.0)
            self:setAnimationTime(self.oilTempAnim, 0.0)
            self:setAnimationTime(self.loadLevelAnim, 0.0)
        end
    end;
end;

function JohnDeereHud:updateTick(dt)
end;

function JohnDeereHud:draw()
end;
